#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test CheckWNs module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import CheckWNs
from Tests.Utilities.GeneralUtils import assertDiracFailsWith, assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.CheckWNs'

# pylint: disable=protected-access


class CheckWNsTestCase(unittest.TestCase):
  """Base class for the Pythia test cases."""

  def setUp(self):
    """set up the objects."""
    self.cwn = CheckWNs({})

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.cwn._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.cwn._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.cwn._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.cwn._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_checkconsistency(self):
    assertDiracSucceeds(self.cwn._checkConsistency(), self)

  def test_checkconsistency_nojob(self):
    assertDiracSucceeds(self.cwn._checkConsistency(None), self)

  def test_checkconsistency_mock_job(self):
    assertDiracSucceeds(self.cwn._checkConsistency(Mock()), self)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""This agent takes care of uploading production logs for Inactive productions.

Get the files by walking the tree from the BaseLogPath option (in the CS, under the Agents sections).
Puts them in tar files that are created in the BasePath(same as BaseLogPath)/LogsTar folder, created if needed.
Deletes the logs once the tar file exists.
Uploads them to the ArchivalSE: defined in Operations, under Transformations/ArchivalSE
Deletes the tar files.
The LFN path is given in the CS, Operations, under Transformations/BaseLogLFN


:since: Nov 2, 2013
:author: sposs
:author: sailer
"""

from __future__ import print_function
from __future__ import absolute_import
from DIRAC.Core.Base.AgentModule import AgentModule
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from DIRAC.Resources.Storage.StorageElement import StorageElementItem as StorageElement

from DIRAC import S_OK, S_ERROR, gLogger

import os
import tarfile
import subprocess

__RCSID__ = "$Id$"

ACTIVE_STATUS = ["Active", 'Completing']


class TarTheProdLogsAgent(AgentModule):
  """Tar the prod logs, and send them to whatever storage element you want."""

  def __init__(self, *args, **kwargs):
    """Constructor."""
    AgentModule.__init__(self, *args, **kwargs)
    self.name = "TarTheProdLogsAgent"
    self.log = gLogger
    self.basepath = ""
    self.baselogpath = ""
    self.transclient = None
    self.ops = None
    self.storageElement = None
    self.baselfn = ""

  def initialize(self):
    """Sets defaults."""
    self.am_setModuleParam("shifterProxy", "ProductionManager")

    self.basepath = self.am_getOption("BasePath", "")
    if not self.basepath:
      return S_ERROR("Missing mandatory option BasePath")

    self.baselogpath = self.am_getOption("BaseLogPath", "")
    if not self.baselogpath:
      return S_ERROR("Missing mandatory option BaseLogPath")

    self.ops = Operations()

    dest_se = self.ops.getValue("Transformations/ArchiveSE", "")
    if not dest_se:
      return S_ERROR("Missing mandatory option ArchiveSE")
    self.storageElement = StorageElement(dest_se)

    baselfn = self.ops.getValue("Transformations/BaseLogLFN", "")
    if not baselfn:
      return S_ERROR("Missing mandatory option Transformations/BaseLogLFN")
    self.baselfn = baselfn

    self.transclient = TransformationClient()

    self.log.info("Running ")
    return S_OK()

  def execute(self):
    """Run it!"""
    res = self.cleanupPrevious()
    if not res["OK"]:
      self.log.error("Failed to clean up previous run:", res["Message"])
      return res

    res = self.getDirectories()
    if not res["OK"]:
      return res

    res = self.getProductionIDs(res["Value"])
    if not res["OK"]:
      return res

    prods = res['Value']
    for prod, files in prods.items():
      res = self.transIsStopped(prod)
      if not res['OK']:
        continue
      if not res["Value"]:
        continue

      res = self.createTarBallAndCleanTheLogs(prod, files)
      if not res["OK"]:
        self.log.error("Could not get the tar ball:", res["Message"])
        continue

      tarBall = res["Value"]
      res = self.uploadToStorage(prod, tarBall)
      if not res["OK"]:
        self.log.error("Failed putting the file to storage:", res["Message"])
      else:
        res = self.cleanTarBall(tarBall)
        if not res["OK"]:
          self.log.error("Failed removing the Tar Ball", res["Message"])

    return S_OK()

  def cleanupPrevious(self):
    """Look for previously created tar files, and try to upload them again.

    Also, create the work dir
    """
    logs_dir = os.path.join(self.basepath, "LogsTars")
    if not os.path.isdir(logs_dir):
      try:
        os.mkdir(logs_dir)
      except OSError:
        return S_ERROR("Could not produce the directory")

    for root, dummy_dirs, files in os.walk(logs_dir):
      if not len(files):
        continue
      prod = root.rstrip("/").split("/")[-1]
      for tfile in files:
        tarballpath = os.path.join(root, tfile)
        res = self.uploadToStorage(prod, tarballpath)
        if not res['OK']:
          self.log.error("Failed to upload again %s to the SE")
          continue
        res = self.cleanTarBall(tarballpath)
        if not res["OK"]:
          self.log.error("Failed to remove the tar ball")

    return S_OK()

  def getDirectories(self):
    """List the directories below the base."""
    final_dirs = {}
    for root, dirs, dummy_files in os.walk(self.baselogpath):
      if root.split("/")[-1] == "LOG" and len(dirs) > 1:
        logDirs = sorted(dirs)  # sort them so we can remove the last one
        del logDirs[-1]
        final_dirs[root] = logDirs

    return S_OK(final_dirs)

  def getProductionIDs(self, directories_and_files):
    """Given input directories, get the prods_dict {prod:files}, where files is sorted per taskID This is used for he tar ball naming."""
    prods_dict = {}
    for path, files in directories_and_files.items():
      f_list = []
      for logfile in files:
        f_name = os.path.join(path, logfile)
        prodid = int(logfile.split("_")[2])
        if prodid not in prods_dict:
          prods_dict[prodid] = []
        f_list.append(f_name)

      prods_dict[prodid] = sorted(f_list, key=self.__sortbyJob)

    return S_OK(prods_dict)

  def __sortbyJob(self, f_name):
    """returns the taskID given a file name.

    Used for the sorting above
    """
    return int(f_name.split("_")[-2])

  def transIsStopped(self, prod):
    """Check from the TS if the prod is Active or not."""
    res = self.transclient.getTransformation(prod)
    if not res['OK']:
      return res

    trans = res["Value"]
    if trans["Status"] in ACTIVE_STATUS:
      return S_OK(False)
    # meaning the prods are neither Active nor Completing
    return S_OK(True)

  def cleanTarBall(self, tarballpath):
    """Physically remove the tar ball that was created to free disk space."""
    try:
      os.unlink(tarballpath)
    except OSError as x:
      return S_ERROR("Failed with %s" % str(x))
    if os.path.exists(tarballpath):
      self.log.error("The tar ball still exists while it should have be removed: ", tarballpath)
    return S_OK()

  def uploadToStorage(self, prod, tarballpath):
    """Put the file to the Storage Element."""
    # FIXME: I don't think this does the right thing at the moment
    final_lfn_path = self.baselfn + "/" + prod
    res = self.storageElement.isFile(final_lfn_path)
    if not res["OK"]:
      return res
    counter = 0
    tarballbasename = os.path.basename(tarballpath)[:-4]
    lfn = final_lfn_path + "/" + tarballbasename + "_%i.tgz" % counter
    while lfn in res['Value']["Successful"]:
      counter = counter + 1
      lfn = final_lfn_path + "/" + tarballbasename + "_%i.tgz" % counter
      res = self.storageElement.isFile(lfn)

    fileDict = {lfn: tarballpath}
    self.log.info("putFile", fileDict)
    res = self.storageElement.putFile(fileDict)
    if res['OK']:
      if res['Value']['Failed']:
        self.log.error("putFile", res['Value']['Failed'])
    else:
      self.log.error("putFile", res['Message'])

    return S_OK()

  def createTarBallAndCleanTheLogs(self, prod, prodFiles):
    """Create and return the path to the tar ball containing all the prod files The file name contains the first and last taskID included.

    Allows easy finding of
    the right tar ball.
    """
    starttaskid = int(prodFiles[0].split("_")[-2])
    lasttaskid = int(prodFiles[-1].split("_")[-2])
    name = os.path.join(self.baselogpath, prod, prod + "_" + starttaskid + "_to_" + lasttaskid + "_logs.tgz")
    try:
      tarFile = tarfile.open(name, "w:gz")
      for fd in prodFiles:
        tarFile.add(fd)
      tarFile.close()
      for fd in prodFiles:
        os.remove(fd)
    except Exception as e:
      return S_ERROR("Failed with %s" % str(e))
    return S_OK(name)

  def tarTheFolders(self, listOfFolders, outputFileName):
    """make a tarBall out of the list of folders."""
    cmd = ['tar', 'cjf', outputFileName, '--remove-files']
    cmd = cmd + listOfFolders
    print("Creating", outputFileName)
    result = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    outAndErr = result.communicate()
    self.log.error("Error from Tar\n%s" % outAndErr[1])

  def removeEmptyFolders(self, basefolder):
    """removes the empty folders below basefolder."""
    cmd = ['find', basefolder, '-type', 'd', '-empty', '-print', '-delete']
    result = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = result.communicate()
    print("Removed folders\n", out)
    if len(err):
      self.log.error("Errors in find\n", err)

# "

  def getLogFolders(self, logDirs):
    """Returns the first and last logFolder, or just one if there is only that."""
    if len(logDirs) == 1:
      return logDirs
    else:
      return [logDirs[0], logDirs[-1]]



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Retrieve the output sandboxes of jobs created using the API, stored in the repository file.

The repository file is defined when creating the :class:`~ILCDIRAC.Interfaces.API.DiracILC.DiracILC` instance.

Options:
  -r repoLocation             Path to repository file
  -O, --Outputdata            retrieve also the outputdata

:since: Mar 24, 2010
:author: sposs
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, exit as dexit
from DIRAC import gLogger

LOG = gLogger.getSubLogger(__name__)


class _Params(object):
  def __init__(self):
    self.outputdata = False
    self.repo = ''

  def setOuputData(self, dummy_opt):
    self.outputdata = True
    return S_OK()

  def setRepo(self, opt):
    self.repo = opt
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch('O', 'Outputdata', 'retrieve also the outputdata', self.setOuputData)
    Script.registerSwitch('r:', 'Repository=', 'repository file to use', self.setRepo)
    Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                        '\nUsage:',
                                        '  %s [option|cfgfile] ...\n' % Script.scriptName]))


@DIRACScript()
def main():
  repoLocation = ''
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine(ignoreErrors=False)
  repoLocation = clip.repo
  if not repoLocation:
    Script.showHelp()
    dexit(1)
  from DIRAC.Interfaces.API.Dirac import Dirac

  dirac = Dirac(True, repoLocation)

  exitCode = 0
  res = dirac.monitorRepository(False)
  if not res['OK']:
    LOG.error("Failed because %s" % res['Message'])
    dexit(1)

  res = dirac.retrieveRepositorySandboxes()
  if not res['OK']:
    LOG.error("Failed because %s" % res['Message'])
    dexit(1)
  if clip.outputdata:
    res = dirac.retrieveRepositoryData()
    if not res['OK']:
      LOG.error("Failed because %s" % res['Message'])
      exit(1)
  dexit(exitCode)


if __name__ == "__main__":
  main()



"""Retrieve CalibrECal from Calibration.txt file."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
Energy_To_Calibrate = sys.argv[2]
Initial_Calibration_Constant = sys.argv[3]
Mean_Or_Calibration_Constant = sys.argv[4]
ECal_Barrel_Or_EndCap = sys.argv[5].lower()

# Mean is the last value for mean of Gaussian fit written to the Calibration.txt file.

Mean = '1.0'

with open(Calibration_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    startOfTheBlock = False
    blockType = ''
    for line in searchlines:
        if startOfTheBlock:
            if 'endcap' in line.lower():
                blockType = 'endcap'
            elif 'barrel' in line.lower():
                blockType = 'barrel'
            else:
                blockType = ''

        if "___" in line:
            startOfTheBlock = True
        else:
            startOfTheBlock = False

        if ECal_Barrel_Or_EndCap == blockType:
            if 'ECal Digi Mean' in line:
                Mean = float(find_between(line, ' : ', ' :'))


if Mean_Or_Calibration_Constant == 'Mean':
    print (Mean)

elif Mean_Or_Calibration_Constant == 'Calibration_Constant':
    print (str(float(Energy_To_Calibrate) * float(Initial_Calibration_Constant) / float(Mean)))

    Calibration_Text = '_____________________________________________________________________________________' + '\n'
    Calibration_Text += 'ECal_Digi_Extract.py retrieving CalibrECal from digitisation program. ' + '\n'
    Calibration_Text += 'Photon Energy To Calibrate                         : ' + str(Energy_To_Calibrate) + ' /GeV \n'
    Calibration_Text += 'Initial Calibration Constant                       : ' + \
        str(Initial_Calibration_Constant) + '\n'
    Calibration_Text += 'CalibrECal Mean                                    : ' + str(Mean) + ' /GeV\n\n'
    Calibration_Text += 'CalibrECal                                         : ' + \
        str(float(Energy_To_Calibrate) * float(Initial_Calibration_Constant) / float(Mean)) + '\n\n'

    if ECal_Barrel_Or_EndCap == 'barrel':
        Calibration_Text += 'CalibrECalBarrel '
    elif ECal_Barrel_Or_EndCap == 'endcap':
        Calibration_Text += 'CalibrECalEndCap '

    Calibration_Text += 'from digitisation program.' + '\n'
    Calibration_Text += 'Photon Energy To Calibrate                          : ' + str(Energy_To_Calibrate) + ' /GeV\n'
    Calibration_Text += 'Initial Calibration Constant                       : ' + \
        str(Initial_Calibration_Constant) + '\n'
    Calibration_Text += 'CalibrECal' + ECal_Barrel_Or_EndCap + \
        ' Mean                              : ' + str(Mean) + ' /GeV \n\n'
    Calibration_Text += 'CalibrECal' + ECal_Barrel_Or_EndCap + '                                   : ' + \
        str(float(Energy_To_Calibrate) * float(Initial_Calibration_Constant) / float(Mean)) + '\n\n'

    with open(Calibration_File_And_Path, 'a') as myfile:
        myfile.write(Calibration_Text)

else:
    print ('Please select Mean or Calibration_Constant to extract.')



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Function to download and untar the applications, called from :mod:`~ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation`

Also installs all dependencies for the applications

:since:  Apr 7, 2010
:author: Stephane Poss
"""
from __future__ import absolute_import
__RCSID__ = "$Id$"

import hashlib
import os
import shutil
import subprocess
import tarfile
from tarfile import TarError
import time


import six.moves.urllib.error
import six.moves.urllib.parse
import six.moves.urllib.request


from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from ILCDIRAC.Core.Utilities.ResolveDependencies import resolveDeps
from ILCDIRAC.Core.Utilities.PrepareLibs import removeLibc, getLibsToIgnore

from ILCDIRAC.Core.Utilities.WasteCPU import wasteCPUCycles


LOG = gLogger.getSubLogger(__name__)


def createLock(lockname):
  """Need to lock the area to prevent 2 jobs to write in the same area."""
  try:
    with open(lockname, "w") as lock:
      lock.write("Locking this directory\n")
  except IOError as e:
    LOG.error("Failed creating lock")
    return S_ERROR("Not allowed to write here: IOError %s" % (str(e)))
  return S_OK()


def checkLockAge(lockname):
  """Check if there is a lock, and in that case deal with it, potentially remove it after n minutes."""
  overwrite = False
  count = 0
  while True:
    if not os.path.exists(lockname):
      break
    count += 1
    LOG.warn("Will wait one minute before proceeding")
    res = wasteCPUCycles(60)
    if not res['OK']:
      continue
    last_touch = time.time()
    try:
      stat = os.stat(lockname)
      last_touch = stat.st_atime
    except OSError as x:
      LOG.warn("File not available: %s %s, assume removed" % (OSError, str(x)))
      break
    loc_time = time.time()
    if loc_time - last_touch > 30 * 60:  # this is where I say the file is too old to still be valid (30 minutes)
      LOG.info("File is %s seconds old" % str(loc_time - last_touch))
      overwrite = True
      res = clearLock(lockname)
      if res['OK']:
        break
    if count > 60:  # We have been waiting for 60 minutes, something is wrong, kill it
      LOG.error("Seems file stat is wrong, assume buggy, will fail installation")
      #overwrite = True
      res = clearLock(lockname)
      return S_ERROR("Buggy lock, removed: %s" % res['OK'])

  return S_OK(overwrite)


def clearLock(lockname):
  """And we need to clear the lock once the operation is done."""
  try:
    os.unlink(lockname)
  except OSError as e:
    LOG.error("Failed cleaning lock: OSError", "%s" % (str(e)))
    return S_ERROR("Failed to clear lock: %s" % (str(e)))
  return S_OK()


def deleteOld(folder_name):
  """Remove directories."""
  LOG.info("Deleting existing version %s" % folder_name)
  if os.path.exists(folder_name):
    if os.path.isdir(folder_name):
      try:
        shutil.rmtree(folder_name)
      except OSError as x:
        LOG.error("Failed deleting %s because :" % (folder_name), "%s %s" % (OSError, str(x)))
    else:
      try:
        os.remove(folder_name)
      except OSError as e:
        LOG.error("Failed deleting %s because :" % (folder_name), "OSError %s" % (str(e)))
  if os.path.exists(folder_name):
    LOG.error("Oh Oh, something was not right, the directory %s is still here" % folder_name)
  return S_OK()


def downloadFile(tarballURL, app_tar, folder_name):
  """Get the file locally."""
  # need to make sure the url ends with /, other wise concatenation below returns bad url
  if tarballURL[-1] != "/":
    tarballURL += "/"

  app_tar_base = os.path.basename(app_tar)
  if tarballURL.find("http://") > -1:
    try:
      LOG.debug("Downloading software", '%s' % (folder_name))
      # Copy the file locally, don't try to read from remote, soooo slow
      # Use string conversion %s%s to set the address, makes the system more stable
      six.moves.urllib.request.urlretrieve("%s%s" % (tarballURL, app_tar), app_tar_base)
    except IOError as err:
      LOG.exception(str(err))
      return S_ERROR('Exception during url retrieve: %s' % str(err))
  else:
    datMan = DataManager()
    resget = datMan.getFile("%s%s" % (tarballURL, app_tar))
    if not resget['OK']:
      LOG.error("File could not be downloaded from the grid")
      return resget
  return S_OK()


def tarMd5Check(app_tar_base, md5sum):
  """Check the tar ball md5 sum."""
  # Tar ball is obtained, need to check its md5 sum
  tar_ball_md5 = ''
  try:
    with open(app_tar_base, 'rb') as myFile:
      tar_ball_md5 = hashlib.md5(myFile.read()).hexdigest()
  except IOError:
    LOG.warn("Failed to get tar ball md5, try without")
    md5sum = ''
  if md5sum and md5sum != tar_ball_md5:
    LOG.error('Hash does not correspond, found %s, expected %s, cannot continue' % (tar_ball_md5, md5sum))
    return S_ERROR("Hash does not correspond")
  return S_OK()


def installDependencies(app, config, areas):
  """install dependencies for application."""
  appName = app[0].lower()
  appVersion = app[1]

  deps = resolveDeps(config, appName, appVersion)
  for dep in deps:
    depapp = [dep["app"], dep["version"]]
    resDep = installInAnyArea(areas, depapp, config)
    if not resDep['OK']:
      return S_ERROR("Failed to install dependency: %s" % str(depapp))

  return S_OK()


def installInAnyArea(areas, app, jobConfig):
  """try to install app in any area of areas."""
  for area in areas:
    resInstall = installSinglePackage(app, jobConfig, area)
    if resInstall['OK']:
      return S_OK()
  # If there was no sucess in the loop, we fail
  return S_ERROR("Failed to install software")


def installSinglePackage(app, jobConfig, area):
  """install some package somewhere, returns S_OK/S_ERROR."""
  LOG.notice('Attempting to install %s_%s for %s in %s' % (app[0], app[1], jobConfig, area))
  curdir = os.getcwd()
  res = installPackage(app, jobConfig, area, curdir)
  if not res['OK']:
    LOG.error('Failed to install software in %s: %s' % (area, res['Message']),
                  '%s_%s' % (app[0], app[1]))
    return S_ERROR("Failed to install here")
  else:
    LOG.info('%s was successfully installed for %s in %s' % (app, jobConfig, area))
    return S_OK()


def getTarBallLocation(app, config, dummy_area):
  """Get the tar ball location."""
  ops = Operations()
  appName = app[0]
  appVersion = app[1]
  appName = appName.lower()
  app_tar = ops.getValue('/AvailableTarBalls/%s/%s/%s/TarBall' % (config, appName, appVersion), '')
  overwrite = ops.getValue('/AvailableTarBalls/%s/%s/%s/Overwrite' % (config, appName, appVersion), False)
  md5sum = ops.getValue('/AvailableTarBalls/%s/%s/%s/Md5Sum' % (config, appName, appVersion), '')
  LOG.info("Looking for application %s%s for config %s:" % (appName, appVersion, config))
  if not app_tar:
    LOG.error('Could not find tar ball for %s %s' % (appName, appVersion))
    return S_ERROR('Could not find tar ball for %s %s' % (appName, appVersion))

  tarballURL = ops.getValue('/AvailableTarBalls/%s/%s/TarBallURL' % (config, appName), '')
  if not tarballURL:
    LOG.error('Could not find tarballURL in CS for %s %s' % (appName, appVersion))
    return S_ERROR('Could not find tarballURL in CS')

  return S_OK([app_tar, tarballURL, overwrite, md5sum])


def install(app, app_tar, tarballURL, overwrite, md5sum, area):
  """Install the software."""
  appName = app[0]
  appVersion = app[1]
  folder_name = app_tar.replace(".tgz", "").replace(".tar.gz", "")
  # jar file does not contain .tgz nor tar.gz so the file name is untouched and folder_name = app_tar
  if appName == "slic":
    folder_name = "%s%s" % (appName, appVersion)

  appli_exists = False
  app_tar_base = os.path.basename(app_tar)

  ###########################################
  # Go where the software is to be installed
  os.chdir(area)
  # We go back to the initial place at any return
  ###########################################
  # Handle the locking
  lockname = folder_name + ".lock"
  # Make sure the lock is not too old, or wait until it's gone
  res = checkLockAge(lockname)
  if not res['OK']:
    LOG.error("Something uncool happened with the lock, will kill installation")
    LOG.error("Message: %s" % res['Message'])
    return S_ERROR("Failed lock checks")

  if 'Value' in res and res['Value']:  # this means the lock file was very old, meaning that the installation failed elsewhere
    overwrite = True

  # Check if the application is here and not to be overwritten
  if os.path.exists(folder_name):
    appli_exists = True
    if not overwrite:
      LOG.info("Folder or file %s found in %s, skipping install !" % (folder_name, area))
      return S_OK([folder_name, app_tar_base])

  # If we are here, it means the application was never installed OR its overwrite flag is true

  # Now lock the area
  res = createLock(lockname)  # This will fail if not allowed to write here
  if not res['OK']:
    LOG.error(res['Message'])
    return res

  # Cleanup old version in case it has to be overwritten (implies it's already here)
  # In particular the jar file of LCSIM
  if appli_exists and overwrite:
    LOG.info("Overwriting %s found in %s" % (folder_name, area))
    res = deleteOld(folder_name)
    if not res['OK']:  # should be always OK for the time being
      clearLock(lockname)
      return res
    # Now application must have been removed

  # If here, the application DOES NOT exist locally: either it was here and the overwrite flag was false and
  # we returned earlier, either it was here and the overwrite flag was true and it was removed, or finally it
  # was never here so here appli_exists=False always

  # Now we can get the files and unpack them

  # Downloading file from url
  res = downloadFile(tarballURL, app_tar, folder_name)
  if not res['OK']:
    clearLock(lockname)
    return res

  # Check that the tar ball is there. Should never happen as download file catches the errors
  if not os.path.exists("%s/%s" % (os.getcwd(), app_tar_base)):
    LOG.error('Failed to download software', '%s' % (folder_name))
    clearLock(lockname)
    return S_ERROR('Failed to download software')

  # FIXME: this is really bad style, suggestion: create 2 private methods that (download a file and check its md5)
  # and (delete the old files and cleanup), then call the download_and_check method once and create a loop
  # with a MAX_TRIES variable that cleans up the old files and tries to download again until MAX_TRIES are used up.
  # The loop is only entered if download/md5check fail and if anything goes wrong in the loop, `continue` is called
  # Could also think about creating a clearLockAndExit method that takes a string (lockname) and return value res
  # and just calls clearLock(lockname) return res. Then replace these double
  # calls in this method with return clearLockAndExit

  # Check that the downloaded file (or existing one) has the right checksum
  res = tarMd5Check(app_tar_base, md5sum)
  if not res['OK']:
    LOG.error("Will try getting the file again, who knows")
    try:  # Remove tar ball that we just got
      os.unlink("%s/%s" % (os.getcwd(), app_tar_base))
    except OSError:
      LOG.error("Failed to clean tar ball, something bad is happening")
    # Clean up existing stuff (if any, in particular the jar file)
    res = deleteOld(folder_name)
    if not res['OK']:  # should be always OK for the time being
      clearLock(lockname)
      return res
    res = downloadFile(tarballURL, app_tar, folder_name)
    if not res['OK']:
      clearLock(lockname)
      return res
    res = tarMd5Check(app_tar_base, md5sum)
    if not res['OK']:
      LOG.error("Hash failed again, something is really wrong, cannot continue.")
      clearLock(lockname)
      return S_ERROR("MD5 check failed")

  if tarfile.is_tarfile(app_tar_base):  # needed because LCSIM is jar file
    app_tar_to_untar = tarfile.open(app_tar_base)
    try:
      app_tar_to_untar.extractall()
    except TarError as e:
      LOG.error("Could not extract tar ball %s because of %s, cannot continue !" % (app_tar_base, str(e)))
      clearLock(lockname)
      return S_ERROR("Could not extract tar ball %s because of %s, cannot continue !" % (app_tar_base, str(e)))
    if folder_name.count("slic"):
      slicname = folder_name
      members = app_tar_to_untar.getmembers()
      fileexample = members[0].name
      basefolder = fileexample.split("/")[0]
      try:
        os.rename(basefolder, slicname)
      except OSError as e:
        LOG.error("Failed renaming slic:", str(e))
        clearLock(lockname)
        return S_ERROR("Could not rename slic directory")
  try:
    dircontent = os.listdir(folder_name)
    if not len(dircontent):
      clearLock(lockname)
      return S_ERROR("Folder %s is empty, considering install as failed" % folder_name)
  except OSError:
    pass

  # Everything went fine, we try to clear the lock
  clearLock(lockname)

  return S_OK([folder_name, app_tar_base])


def check(app, area, res_from_install):
  """Now that the tar ball is here, we need to check that all is there."""
  ###########################################
  # Go where the software is to be installed
  os.chdir(area)
  # We go back to the initial place either at the end of the installation or at any error
  ###########################################

  basefolder = res_from_install[0]
  if os.path.isfile(basefolder):
    # This is the case of LCSIM that's a jar file
    return S_OK([basefolder])

  if os.path.exists(os.path.join(basefolder, 'md5_checksum.md5')):
    with open(os.path.join(basefolder, 'md5_checksum.md5'), 'r') as md5file:
      for line in md5file:
        line = line.rstrip()
        md5sum, fin = line.split()
        if fin == '-' or fin.count("md5_checksum.md5"):
          continue
        found_lib_to_ignore = False
        for lib in getLibsToIgnore():
          if fin.count(lib):
            found_lib_to_ignore = True
        if found_lib_to_ignore:
          continue
        fin = os.path.join(basefolder, fin.replace("./", ""))
        if not os.path.exists(fin):
          LOG.error("File missing :", fin)
          return S_ERROR("Incomplete install: The file %s is missing" % fin)
        fmd5 = ''
        try:
          # do not change to with open, messes up test
          fmd5 = hashlib.md5(open(fin, 'rb').read()).hexdigest()
        except IOError:
          LOG.error("Failed to compute md5 sum")
          return S_ERROR("Failed to compute md5 sum")
        if md5sum != fmd5:
          LOG.error("File has wrong checksum :", fin)
          LOG.error("Found %s, expected %s" % (fmd5, md5sum))
          return S_ERROR("Corrupted install: File %s has a wrong sum" % fin)
  else:
    LOG.warn("The application does not come with md5 checksum file:", app)

  return S_OK([basefolder])


def configure(app, area, res_from_check):
  """Configure our applications: set the proper env variables."""
  ###########################################
  # Go where the software is to be installed
  os.chdir(area)
  # We go back to the initial place either at the end of the installation or at any error
  ###########################################

  appName = app[0].lower()
  # Set env variables
  basefolder = res_from_check[0]
  removeLibc(os.path.join(os.getcwd(), basefolder, "LDLibs"))
  libFolder = os.path.join(os.getcwd(), basefolder, "lib")
  if os.path.isdir(libFolder):
    removeLibc(os.path.join(libFolder))
    addFolderToLdLibraryPath(libFolder)

  if appName == "slic":
    configureSlic(basefolder)
  elif appName == "root":
    configureRoot(basefolder)
  elif appName == 'java':
    configureJava(basefolder)
  elif appName == "lcio":
    res = checkJava()
    if not res['OK']:
      return res
    configureLCIO(basefolder)
  elif appName in ["lcsim", 'stdhepcutjava']:
    res = checkJava()
    if not res['OK']:
      return res
  return S_OK()


def clean(area, res_from_install):
  """After install, clean the tar balls and go back to initial directory."""
  ###########################################
  # Go where the software is to be installed
  os.chdir(area)
  # We go back to the initial place at the end
  ###########################################
  app_tar_base = res_from_install[1]
  # remove now useless tar ball
  if os.path.exists("%s/%s" % (os.getcwd(), app_tar_base)):
    if app_tar_base.find(".jar") < 0:
      try:
        os.unlink(app_tar_base)
      except OSError as e:
        LOG.error("Could not remove tar ball:", str(e))
  return S_OK()


def remove():
  """For the moment, this is done in :mod:`~ILCDIRAC.Core.Utilities.RemoveSoft`"""
  pass


def checkJava():
  """Check if JAVA is available locally."""
  args = ['java', "-Xmx1536m", "-Xms256m", "-version"]
  try:
    res = subprocess.check_call(args)
    if res:
      return S_ERROR("Something is wrong with Java")
  except subprocess.CalledProcessError:
    LOG.error("Java was not found on this machine, cannot proceed")
    return S_ERROR("Java was not found on this machine, cannot proceed")
  return S_OK()


def configureSlic(basefolder):
  """sets environment variables for SLIC."""
  slicfolder = os.path.join(os.getcwd(), basefolder)
  os.environ['SLIC_DIR'] = slicfolder
  slicv = ''
  lcddv = ''
  xercesv = ''
  try:
    slicv = os.listdir(os.path.join(slicfolder, 'packages/slic/'))[0]
    lcddv = os.listdir(os.path.join(slicfolder, 'packages/lcdd/'))[0]
    if os.path.exists(os.path.join(slicfolder, 'packages/xerces/')):
      xercesv = os.listdir(os.path.join(slicfolder, 'packages/xerces/'))[0]
  except OSError:
    return S_ERROR("Could not resolve slic env variables, folder content does not match usual pattern")
  # for mem in members:
  #  if mem.name.find('/packages/slic/')>0:
  #    slicv = mem.name.split("/")[3]
  #  if mem.name.find('/packages/lcdd/')>0:
  #    lcddv = mem.name.split("/")[3]
  #  if mem.name.find('/packages/xerces/')>0:
  #    xercesv = mem.name.split("/")[3]
  if slicv:
    os.environ['SLIC_VERSION'] = slicv
  if xercesv:
    os.environ['XERCES_VERSION'] = xercesv
  if lcddv:
    os.environ['LCDD_VERSION'] = lcddv


def configureRoot(basefolder):
  """Sets environment variables for root."""
  #members = app_tar_to_untar.getmembers()
  #fileexample = members[0].name
  # fileexample.split("/")[0]
  rootFolder = os.path.join(os.getcwd(), basefolder)
  rootLibFolder = os.path.join(rootFolder, "lib")
  rootBinFolder = os.path.join(rootFolder, "bin")
  os.environ['ROOTSYS'] = rootFolder
  addFolderToLdLibraryPath(rootLibFolder)
  os.environ['PATH'] = ":".join((rootBinFolder, os.environ['PATH']))
  os.environ['PYTHONPATH'] = ":".join((rootLibFolder, os.environ["PYTHONPATH"]))


def configureJava(basefolder):
  """sets the environment variables for java."""
  binFolder = os.path.join(os.getcwd(), basefolder, "bin")
  libFolder = os.path.join(os.getcwd(), basefolder, "lib")
  os.environ['PATH'] = ":".join((binFolder, os.environ['PATH']))
  addFolderToLdLibraryPath(libFolder)


def configureLCIO(basefolder):
  """sets the environment variables for LCIO."""
  lcioFolder = os.path.join(os.getcwd(), basefolder)
  os.environ['LCIO'] = lcioFolder
  os.environ['PATH'] = ":".join((os.path.join(lcioFolder, "bin"), os.environ['PATH']))


def addFolderToLdLibraryPath(folder):
  """insert folder to os.environ variables."""
  if 'LD_LIBRARY_PATH' in os.environ:
    os.environ['LD_LIBRARY_PATH'] = ":".join((folder, os.environ['LD_LIBRARY_PATH']))
  else:
    os.environ['LD_LIBRARY_PATH'] = folder


def installPackage(app, config, area, curdir):
  """Installs the package app."""
  appName = app[0]
  appVersion = app[1]
  res = getTarBallLocation(app, config, area)
  if not res['OK']:
    LOG.error("Could not install software/dependency %s %s: %s" % (appName, appVersion, res['Message']))
    return S_ERROR('Failed to install software')
  app_tar, tarballURL, overwrite, md5sum = res['Value']

  res = install(app, app_tar, tarballURL, overwrite, md5sum, area)
  os.chdir(curdir)
  if not res['OK']:
    LOG.error("Could not install software/dependency %s %s: %s" % (appName, appVersion, res['Message']))
    return S_ERROR('Failed to install software')
  res_from_install = res['Value']

  res = check(app, area, res_from_install)
  os.chdir(curdir)
  if not res['OK']:
    LOG.error("Failed to check software/dependency %s %s" % (appName, appVersion))
    return S_ERROR('Failed to check integrity of software')
  res_from_check = res['Value']

  res = configure(app, area, res_from_check)
  os.chdir(curdir)
  if not res['OK']:
    LOG.error("Failed to configure software/dependency %s %s" % (appName, appVersion))
    return S_ERROR('Failed to configure software')

  res = clean(area, res_from_install)
  os.chdir(curdir)
  if not res['OK']:
    LOG.error("Failed to clean useless tar balls, deal with it: %s %s" % (appName, appVersion))

  os.chdir(curdir)
  LOG.notice("Successfully installed %s %s in %s" % (appName, appVersion, area))
  return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the WhizardAnalysis module."""

from __future__ import absolute_import
import unittest
import os
from mock import call, mock_open, patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Workflow.Modules.WhizardAnalysis import WhizardAnalysis
from Tests.Utilities.GeneralUtils import assertEqualsImproved, \
    assertDiracFailsWith, assertDiracSucceeds, assertDiracSucceedsWith_equals, \
    assertMockCalls
from Tests.Utilities.FileUtils import FileUtil
from six.moves import zip

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.WhizardAnalysis'

# pylint: disable=too-many-public-methods,missing-docstring


class WhizardAnalysisTestCase(unittest.TestCase):
  """Base class for the WhizardAnalysis test cases."""

  def setUp(self):
    self.wha = WhizardAnalysis()
    self.wha.platform = 'myTestPlatform'
    self.wha.applicationLog = '/mydir/AppLog/test.txt'
    self.wha.workflowStatus = S_OK()
    self.wha.stepStatus = S_OK()

  def test_obtainprocesslist_getfile_fails(self):
    datman_mock = Mock()
    datman_mock.getFile.return_value = S_ERROR('network_error_testME')
    operations_mock = Mock()
    operations_mock.getValue.return_value = 'mytestproclist'
    self.wha.ops = operations_mock
    self.wha.datMan = datman_mock
    with patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=False)):
      res = self.wha.obtainProcessList()
      assertDiracFailsWith(res, 'network_error_testme', self)
      datman_mock.getFile.assert_called_once_with('mytestproclist')
      operations_mock.getValue.assert_called_once_with('/ProcessList/Location',
                                                        '')

  def test_applicationSpecificInputs(self):
    self.wha.energy = '42189GeV'
    self.wha.RandomSeed = None
    self.wha.jobID = 840
    self.wha.workflow_commons['IS_PROD'] = True
    self.wha.workflow_commons['PRODUCTION_ID'] = '874'
    self.wha.workflow_commons['JOB_ID'] = '783'
    self.wha.workflow_commons['ProductionOutputData'] = 'myoutputfile_dontuseme;output_gen_test.exe'
    self.wha.step_commons['SusyModel'] = 2
    self.wha.step_commons['InputFile'] = '/ignore/this/inpfTest.me'
    self.wha.steeringparameters = 'testparam=9842;illegalparam;another=one'
    self.wha.willCut = False
    result = self.wha.applicationSpecificInputs()
    assertDiracSucceeds(result, self)
    assertEqualsImproved(('42189GeV', 874783, self.wha.parameters['SEED'], 2,
                           'inpfTest.me', '9842', 'one', False,
                           'output_gen_test.exe'),
                          (self.wha.parameters['ENERGY'], self.wha.RandomSeed,
                            self.wha.RandomSeed, self.wha.susymodel,
                            self.wha.SteeringFile, self.wha.parameters['testparam'],
                            self.wha.parameters['another'], self.wha.getProcessInFile,
                            self.wha.OutputFile), self)

  def test_applicationSpecificInputs_1(self):
    self.wha.RandomSeed = 'myrandseed'
    self.wha.SteeringFile = 'whizard.in'
    self.wha.steeringparameters = ''
    self.wha.OptionsDictStr = 'blab'
    with patch('%s.os.rename' % MODULE_NAME) as rename_mock, \
         patch('%s.eval' % MODULE_NAME, new=Mock(side_effect=IOError('dont use eval...'))):
      result = self.wha.applicationSpecificInputs()
      assertDiracFailsWith(
          result, 'could not convert string to dictionary for optionsdict', self)
      rename_mock.assert_called_once_with('whizard.in', 'whizardnew.in')
      assertEqualsImproved(('myrandseed', 'whizardnew.in', 0),
                            (self.wha.RandomSeed, self.wha.SteeringFile,
                              self.wha.susymodel), self)

  def test_applicationSpecificInputs_2(self):
    self.wha.RandomSeed = '9803245'
    self.wha.SteeringFile = 'testme'
    self.wha.steeringparameters = ''
    self.wha.OptionsDictStr = 'well_formed_dict'
    self.wha.GenLevelCutDictStr = 'bla'

    def replace_eval(string_to_parse):
      if string_to_parse == 'well_formed_dict':
        return {'process_input': {'sqrts': 'mytestenergywithroots'},
                 'random_entry': 'testme'}
      else:
        raise IOError('dont use eval!')
    with patch('%s.os.rename' % MODULE_NAME) as rename_mock, \
         patch('%s.eval' % MODULE_NAME, new=Mock(side_effect=replace_eval)):
      result = self.wha.applicationSpecificInputs()
      assertDiracFailsWith(
          result, 'could not convert the generator level cuts back to dictionary',
          self)
      self.assertFalse(rename_mock.called)
      assertEqualsImproved(('9803245', 'testme', 0,
                              {'integration_input': {'seed': 9803245},
                                'process_input': {'sqrts': 'mytestenergywithroots'},
                                'random_entry': 'testme'}, 'mytestenergywithroots'),
                            (self.wha.RandomSeed, self.wha.SteeringFile, self.wha.susymodel,
                              self.wha.optionsdict, self.wha.energy), self)

  def test_applicationSpecificInputs_3(self):
    self.wha.RandomSeed = 'myrandseed'
    self.wha.SteeringFile = 'testme'
    self.wha.steeringparameters = ''
    self.wha.OptionsDictStr = 'blab'
    self.wha.GenLevelCutDictStr = 'bla'
    self.wha.workflow_commons['IS_DBD_GEN_PROD'] = True
    self.wha.workflow_commons['PRODUCTION_ID'] = 348
    self.wha.workflow_commons['JOB_ID'] = 742
    self.wha.workflow_commons['ProductionOutputData'] = 'abc;ignore_me'
    with patch('%s.os.rename' % MODULE_NAME) as rename_mock, \
         patch('%s.eval' % MODULE_NAME, new=Mock(side_effect=[{'integration_input': {'seed': 'otherseed'}, 'random_entry': 'testme'}, {'something': 'else'}])):
      result = self.wha.applicationSpecificInputs()
      assertDiracSucceeds(result, self)
      self.assertFalse(rename_mock.called)
      assertEqualsImproved(({'integration_input': {'seed': 'otherseed'},
                                'random_entry': 'testme'}, {'something': 'else'},
                              'abc'), (self.wha.optionsdict, self.wha.genlevelcuts,
                                         self.wha.OutputFile), self)

  def test_runit(self):
    set_default_values(self.wha)
    self.wha.workflow_commons['Info'] = {}  # Has to stay for the refactoring
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': False,
                    'my/test/soft/dir/LesHouches_slsqhh.msugra_1.in': True,
                    'Whizard_myTestV1_Run_testStep12.sh': False,
                    'mytestAppLOg': False, 'whizard.out': True,
                    'my/test/soft/dir/myTestGotFile': True,
                    'myTestEvents.001.stdhep': True}

    def exists_replace(key):
      value = exists_dict[key]
      if key == 'mytestAppLOg':
        exists_dict['mytestAppLOg'] = True
      return value
    file_contents = [[], ['some_logging_ignoreme',
                            '! Event sample corresponds to luminosity 92847',
                            'Event generation finished. Success!'], [
        'a 12.1 489.3 b 91.2   843            "']]
    genmodel_mock = Mock()
    genmodel_mock.hasModel.return_value = S_OK()
    genmodel_mock.getFile.return_value = S_OK('myTestGotFile')
    self.wha.genmodel = genmodel_mock
    whizopts_mock = Mock()
    whizopts_mock.changeAndReturn.return_value = S_OK()
    whizopts_mock.toWhizardDotIn.return_value = S_OK('')
    whizopts_mock.getAsDict.return_value = S_OK({'process_input':
                                                   {'process_id': '123 843'},
                                                   'some': 'dict'})
    whiz_options_mock = Mock(return_value=whizopts_mock)
    getops_mock = Mock()
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    expected_glob = [call('myTestEvents*.stdhep')]
    expected_shell = [call(0, 'sh -c "./Whizard_myTestV1_Run_testStep12.sh"',
                            callbackFunction=self.wha.redirectLogOutput,
                            bufferLimit=209715200)]
    expected_copy = []
    expected_genmodel = [
        call.hasModel('mytestMODEL'), call.getFile('mytestMODEL'),
        call.getFile('mytestMODEL'), call.getFile('mytestMODEL')]
    expected_getops = []
    expected_remove = []
    expected_chmod = [call('Whizard_myTestV1_Run_testStep12.sh', 0o755)]
    expected_appstat = [call('Whizard myTestV1 step testStep12'),
                         call('Whizard myTestV1 Successful')]
    expected_exists = [call('LesHouches.msugra_1.in'),
                        call('my/test/soft/dir/LesHouches_slsqhh.msugra_1.in'),
                        call('my/test/soft/dir/myTestGotFile'),
                        call('Whizard_myTestV1_Run_testStep12.sh'),
                        call('mytestAppLOg'), call('mytestAppLOg'),
                        call('whizard.out'), call('myTestEvents.001.stdhep')]
    expected_rename = [
        call('myoutputfile_123.stdhep', 'mytestwhizardOutputFile_1.stdhep'),
        call('myotherfile_121.stdhep', 'mytestwhizardOutputFile_2.stdhep'),
        call('lastfile_9824.stdhep', 'mytestwhizardOutputFile_3.stdhep')]
    expected_calls = [
        [call('#!/bin/sh \n'), call('#####################################################################\n'),
         call('# Dynamically generated script to run a production or analysis job. #\n'),
         call('#####################################################################\n'),
         call('declare -x PATH=my/test/soft/dir:$PATH\n'),
         call('declare -x LD_LIBRARY_PATH=my/test/soft/dir/lib:my/lib/path\n'),
         call('env | sort >> localEnv.log\n'),
         call('echo =============================\n'),
         call('echo Printing content of whizard.in \n'), call('cat whizard.in\n'),
         call('echo =============================\n'),
         call('cp  my/test/soft/dir/whizard.mdl ./\n'),
         call('cp my/test/soft/dir/myTestGotFile ./LesHouches.msugra_1.in\n'),
         call('ln -s LesHouches.msugra_1.in fort.71\n'),
         call('cp mygridfiles/folder/My99TestTeVFile.energy/gridfile1.txt ./\n'),
         call('cp mygridfiles/folder/My99TestTeVFile.energy/cool/gridfile2.ppt ./\n'),
         call('cp mygridfiles/folder/My99TestTeVFile.energy/My99TestTeVFile.energy ./\n'),
         call('cp my/test/soft/dir/whizard.prc ./\n'),
         call('echo =============================\n'),
         call('echo Printing content of whizard.prc \n'),
         call('cat whizard.prc\n'), call('echo =============================\n'),
         call('whizard --process_input \'process_id =\"myTestEvents\"\' --simulation_input \'write_events_file = \"myTestEvents\"\'  extraTestCLIargs \n'),
         call('declare -x appstatus=$?\n'), call('exit $appstatus\n')], [], []]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'My99TestTeVFile.energy'], ['gridfile1.txt', 'cool/gridfile2.ppt', 'My99TestTeVFile.energy']])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=exists_replace)) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(return_value=True)) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as open_mock, \
         patch('%s.WhizardAnalysis.makeWhizardDotCut1' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK())) as shell_mock, \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['myoutputfile_123.stdhep', 'myotherfile_121.stdhep', 'lastfile_9824.stdhep'])) as glob_mock, \
         patch('%s.os.rename' % MODULE_NAME) as rename_mock:
      open_mock.side_effect = (h for h in handles)
      result = self.wha.runIt()
      assertDiracSucceedsWith_equals(result, {
          'OutputFile': 'mytestwhizardOutputFile'}, self)
      assertMockCalls(open_mock, [('Whizard_myTestV1_Run_testStep12.sh', 'w'), 'mytestAppLOg',
                                    ('whizard.out', 'r')], self)
      check_runit_for_parameters(self, whiz_options_mock, getops_mock,
                                  genmodel_mock, expected_calls, handles,
                                  appstat_mock, exists_mock, rename_mock,
                                  chmod_mock, glob_mock, shell_mock, copy_mock,
                                  remove_mock, open_mock, expected_appstat,
                                  expected_exists, expected_rename,
                                  expected_chmod, expected_glob,
                                  expected_shell, expected_copy, expected_genmodel,
                                  expected_getops, expected_remove)
      assertEqualsImproved(self.wha.workflow_commons['Info'], {
          'xsection': {'843': {'xsection': 12.1, 'err_xsection': 489.3,
                                 'fraction': 91.2}}}, self)
      assertEqualsImproved(self.wha.workflow_commons['Luminosity'], 92847,
                            self)
      whizopts_mock.getAsDict.assert_called_once_with()

  def test_runit_no_platform(self):
    self.wha.platform = None
    assertDiracFailsWith(self.wha.runIt(), 'no ilc platform selected', self)

  def test_runit_no_logfile(self):
    self.wha.applicationLog = None
    assertDiracFailsWith(self.wha.runIt(), 'no log file provided', self)

  def test_runit_workflowstatus_bad(self):
    self.wha.workflowStatus = S_ERROR('workflow_err_testme')
    assertDiracSucceedsWith_equals(
        self.wha.runIt(), 'Whizard should not proceed as previous step did not end properly',
        self)

  def test_runit_stepstatus_bad(self):
    self.wha.stepStatus = S_ERROR('step_err_testme')
    assertDiracSucceedsWith_equals(
        self.wha.runIt(), 'Whizard should not proceed as previous step did not end properly',
        self)

  def test_runit_getsoftwarefolder_fails(self):
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(return_value=S_ERROR('get_software_network_test_error'))) as getsoft_mock, \
        patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock:
      assertDiracFailsWith(self.wha.runIt(), 'get_software_network_test_error',
                            self)
      appstat_mock.assert_called_once_with('Failed finding software')
      getsoft_mock.assert_called_once_with('myTestPlatform', 'whizard', '')

  def test_runit_getdependencies_fails(self):
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('test/dep/folder/'), S_ERROR('something_fails_test')])) as getsoft_mock, \
        patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)) as removlib_mock, \
        patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')) as getlibs_mock, \
        patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'testdep1': True, 'app': 'mytestdep', 'version': '4.2'}, {'failsomethingonthisdep': True, 'app': 'faulty_dep_testme', 'version': 'invalid'}])) as resolvdep_mock, \
        patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as setappstatus_mock:
      assertDiracFailsWith(self.wha.runIt(), 'something_fails_test', self)
      setappstatus_mock.assert_called_once_with('Failed finding software')
      getlibs_mock.assert_called_once_with('myTestPlatform', 'whizard', '')
      resolvdep_mock.assert_called_once_with('myTestPlatform', 'whizard', '')
      removlib_mock.assert_called_once_with('my/test/soft/dir/lib')
      assertMockCalls(getsoft_mock, [('myTestPlatform', 'whizard', ''),
                                       ('myTestPlatform', 'mytestdep', '4.2'),
                                       ('myTestPlatform', 'faulty_dep_testme', 'invalid')], self)

  def test_runit_whizardin_fails(self):
    self.wha.useGridFiles = True
    self.wha.getProcessInFile = True
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = False
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock:
      assertDiracFailsWith(self.wha.runIt(),
                            'error while resolving whizard input file', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      exists_mock.assert_called_once_with('list.txt')
      appstat_mock.assert_called_once_with('Whizard input file was not found')
      assertEqualsImproved(EXPECTED_TEST_ENVIRON, os.environ, self)

  def test_runit_getprocesslist_fails(self):
    self.wha.useGridFiles = True
    self.wha.getProcessInFile = True
    getops_mock = Mock()
    getops_mock.getValue.return_value = ''
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = False
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(return_value=True)) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock:
      assertDiracFailsWith(self.wha.runIt(), 'no process list found', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      self.assertFalse(exists_mock.called)
      appstat_mock.assert_called_once_with('Failed getting processlist')

  def test_runit_leshouchesfile_missing(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': False,
                    'my/test/soft/dir/myTestGotFile': False}
    self.wha.useGridFiles = True
    self.wha.getProcessInFile = True
    self.wha.Model = True
    self.wha.susymodel = False
    genmodel_mock = Mock()
    genmodel_mock.hasModel.return_value = S_OK()
    genmodel_mock.getFile.return_value = S_OK('myTestGotFile')
    self.wha.genmodel = genmodel_mock
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock:
      assertDiracFailsWith(self.wha.runIt(), 'the leshouches file was not found',
                            self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      assertEqualsImproved(len(exists_mock.mock_calls), 3, self)
      appstat_mock.assert_called_once_with('LesHouches file missing')
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      genmodel_mock.hasModel.assert_called_once_with(True)
      assertMockCalls(genmodel_mock.getFile, [True, True], self)

  def test_runit_model_undefined(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': False}
    self.wha.useGridFiles = True
    self.wha.getProcessInFile = True
    self.wha.Model = True
    self.wha.susymodel = False
    genmodel_mock = Mock()
    genmodel_mock.hasModel.return_value = S_ERROR()
    genmodel_mock.getFile.return_value = S_OK('myTestGotFile')
    self.wha.genmodel = genmodel_mock
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock:
      assertDiracFailsWith(self.wha.runIt(), 'no model true defined', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      assertEqualsImproved(len(exists_mock.mock_calls), 2, self)
      appstat_mock.assert_called_once_with('Model undefined')
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      genmodel_mock.hasModel.assert_called_once_with(True)
      self.assertFalse(genmodel_mock.called)

  def test_runit_copy_fails(self):
    exists_dict = {'list.txt': True}
    self.wha.useGridFiles = True
    self.wha.getProcessInFile = True
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(side_effect=EnvironmentError('some_err_msg_test'))):
      assertDiracFailsWith(self.wha.runIt(),
                            'failed to obtain mywhizardtestfile.in', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      exists_mock.assert_called_once_with('list.txt')
      appstat_mock.assert_called_once_with('Failed getting whizard.in file')

  def test_runit_changeandreturn_fails(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True}
    self.wha.Model = 'mytestMODEL'
    self.wha.getProcessInFile = True
    self.wha.optionsdict = 9834
    self.wha.useGridFiles = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    changeandret_mock = Mock()
    changeandret_mock.changeAndReturn.return_value = S_ERROR('changeandret_err_testme')
    whiz_options_mock = Mock(return_value=changeandret_mock)
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock):
      assertDiracFailsWith(self.wha.runIt(), 'changeandret_err_testme', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      assertMockCalls(exists_mock, ['list.txt', 'LesHouches.msugra_1.in'], self)
      self.assertFalse(appstat_mock.called)
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      self.assertFalse(genmodel_mock.called)
      changeandret_mock.changeAndReturn.assert_called_once_with(9834)

  def test_runit_prepareWhizardFile_fails(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True}
    self.wha.Lumi = 'mytestluminosity'
    self.wha.Model = True
    self.wha.NumberOfEvents = 'myTestNumber'
    self.wha.RandomSeed = 'notRandomTestme'
    self.wha.energy = '100TestTeV'
    self.wha.evttype = 'myTestEvent'
    self.wha.getProcessInFile = True
    self.wha.optionsdict = False
    self.wha.susymodel = False
    self.wha.template = False
    self.wha.useGridFiles = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.prepareWhizardFile' % MODULE_NAME, new=Mock(return_value=S_ERROR('preparewhiz_no_template_test_err'))) as preparefile_mock:
      assertDiracFailsWith(self.wha.runIt(),
                            'something went wrong with whizard.in file generation',
                            self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      exists_mock.assert_any_call('list.txt')
      exists_mock.assert_any_call('LesHouches.msugra_1.in')
      assertEqualsImproved(len(exists_mock.mock_calls), 2, self)
      appstat_mock.assert_called_once_with(
          'Whizard: something went wrong with input file generation')
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      self.assertFalse(genmodel_mock.called)
      preparefile_mock.assert_called_once_with(
          'whizardnew.in', 'myTestEvent', '100TestTeV', 'notRandomTestme',
          'myTestNumber', 'mytestluminosity', 'whizard.in')

  def test_runit_prepareWhizardFileTemplate_fails(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True}
    self.wha.Model = True
    self.wha.SteeringFile = 'templateTestme123'
    self.wha.evttype = 'myTestEvent'
    self.wha.getProcessInFile = True
    self.wha.optionsdict = False
    self.wha.parameters = 'myTESTParams'
    self.wha.susymodel = False
    self.wha.template = True
    self.wha.useGridFiles = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.prepareWhizardFileTemplate' % MODULE_NAME, new=Mock(return_value=S_ERROR('preparewhiz_no_template_test_err'))) as preparetemplate_mock:
      assertDiracFailsWith(
          self.wha.runIt(), 'something went wrong with whizard.in file generation',
          self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      exists_mock.assert_any_call('list.txt')
      exists_mock.assert_any_call('LesHouches.msugra_1.in')
      assertEqualsImproved(len(exists_mock.mock_calls), 2, self)
      appstat_mock.assert_called_once_with(
          'Whizard: something went wrong with input file generation')
      copy_mock.assert_called_once_with(
          'my/test/soft/dir/mywhizardTestFile.in', './whizardnew.in')
      self.assertFalse(genmodel_mock.called)
      preparetemplate_mock.assert_called_once_with('whizardnew.in', 'myTestEvent',
                                                   'myTESTParams', 'whizard.in')

  def test_runit_makecut1_fails(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True,
                    'Whizard_myTestV1_Run_testStep12.sh': True}
    set_default_values(self.wha)
    self.wha.genlevelcuts = {'some_entry': True,
                              'this_dict_is_not_empty': True}
    self.wha.getProcessInFile = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    whizopts_mock = Mock()
    whizopts_mock.changeAndReturn.return_value = S_OK()
    whizopts_mock.toWhizardDotIn.return_value = S_OK('mytestprocessfindme')
    whiz_options_mock = Mock(return_value=whizopts_mock)
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    expected_calls = [
        [call('#!/bin/sh \n'), call('#####################################################################\n'),
         call('# Dynamically generated script to run a production or analysis job. #\n'),
         call('#####################################################################\n'),
         call('declare -x PATH=my/test/soft/dir:$PATH\n'),
         call('declare -x LD_LIBRARY_PATH=my/test/soft/dir/lib:my/lib/path\n'),
         call('env | sort >> localEnv.log\n'),
         call('echo =============================\n'),
         call('echo Printing content of whizard.in \n'),
         call('cat whizard.in\n'),
         call('echo =============================\n'),
         call('cp  my/test/soft/dir/whizard.mdl ./\n'),
         call('ln -s LesHouches.msugra_1.in fort.71\n'),
         call('cp file1.grb ./\n'), call('cp otherfile.grb ./\n'),
         call('cp testfile.grc ./\n'),
         call('cp my/test/soft/dir/whizard.prc ./\n')]]
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(return_value=True)) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.WhizardAnalysis.makeWhizardDotCut1' % MODULE_NAME, new=Mock(return_value=S_ERROR())):
      # FIXME: makeWhizardDotCut1 currently cant fail (although it operates on files?!)
      result = self.wha.runIt()
      assertDiracFailsWith(result, 'could not create the cut1 file', self)
      open_mock.assert_called_once_with('Whizard_myTestV1_Run_testStep12.sh', 'w')
      rename_mock = Mock()
      chmod_mock = Mock()
      glob_mock = Mock()
      shell_mock = Mock()
      handles = [open_mock()]
      expected_appstat = []
      expected_rename = []
      expected_chmod = []
      expected_glob = []
      expected_shell = []
      expected_copy = [call('my/test/soft/dir/mywhizardTestFile.in',
                              './whizardnew.in')]
      expected_genmodel = []
      expected_getops = [call.getValue('/ProcessList/Location', '')]
      expected_remove = [call('Whizard_myTestV1_Run_testStep12.sh')]
      expected_exists = [
          call('list.txt'), call('LesHouches.msugra_1.in'),
          call('Whizard_myTestV1_Run_testStep12.sh')]
      check_runit_for_parameters(self, whiz_options_mock, getops_mock,
                                  genmodel_mock, expected_calls, handles,
                                  appstat_mock, exists_mock, rename_mock,
                                  chmod_mock, glob_mock, shell_mock, copy_mock,
                                  remove_mock, open_mock, expected_appstat,
                                  expected_exists, expected_rename, expected_chmod,
                                  expected_glob, expected_shell, expected_copy,
                                  expected_genmodel, expected_getops,
                                  expected_remove)

  def test_runit_noapplog_created(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True,
                    'Whizard_myTestV1_Run_testStep12.sh': True,
                    'mytestAppLOg': True}

    def replace_exists(path):
      result = exists_dict[path]
      if path == 'mytestAppLOg':
        exists_dict[path] = False
      return result
    self.wha.Model = 'mytestMODEL'
    self.wha.STEP_NUMBER = 'testStep12'
    self.wha.applicationLog = 'mytestAppLOg'
    self.wha.applicationVersion = 'myTestV1'
    self.wha.debug = False
    self.wha.evttype = 'myTestEvents'
    self.wha.extraCLIarguments = 'extraTestCLIargs'
    self.wha.genlevelcuts = False
    self.wha.getProcessInFile = True
    self.wha.ignoreapperrors = False
    self.wha.optionsdict = 9834
    self.wha.useGridFiles = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    whizopts_mock = Mock()
    whizopts_mock.changeAndReturn.return_value = S_OK()
    whizopts_mock.toWhizardDotIn.return_value = S_OK('mytestprocessfindme')
    whiz_options_mock = Mock(return_value=whizopts_mock)
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=replace_exists)) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(return_value=True)) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open()) as open_mock, \
         patch('%s.WhizardAnalysis.makeWhizardDotCut1' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_ERROR())) as shell_mock:
      # FIXME: makeWhizardDotCut1 currently cant fail (although it operates on files?!)
      assertDiracFailsWith(self.wha.runIt(), 'whizard did not produce the expected log', self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      assertMockCalls(exists_mock, ['list.txt', 'LesHouches.msugra_1.in',
                                      'Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg',
                                      'mytestAppLOg'], self)
      assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], self)
      assertMockCalls(appstat_mock, ['Whizard myTestV1 step testStep12',
                                       'whizard failed terribly, you are doomed!'], self)
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      self.assertFalse(genmodel_mock.called)
      whizopts_mock.changeAndReturn.assert_called_once_with(9834)
      whizopts_mock.toWhizardDotIn.assert_called_once_with('whizard.in')
      assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], self)
      open_mock.assert_called_once_with('Whizard_myTestV1_Run_testStep12.sh', 'w')
      assertMockCalls(open_mock().write, [
          '#!/bin/sh \n', '#####################################################################\n',
          '# Dynamically generated script to run a production or analysis job. #\n',
          '#####################################################################\n',
          'declare -x PATH=my/test/soft/dir:$PATH\n',
          'declare -x LD_LIBRARY_PATH=my/test/soft/dir/lib:my/lib/path\n', 'env | sort >> localEnv.log\n',
          'echo =============================\n', 'echo Printing content of whizard.in \n', 'cat whizard.in\n',
          'echo =============================\n', 'cp  my/test/soft/dir/whizard.mdl ./\n',
          'ln -s LesHouches.msugra_1.in fort.71\n', 'cp file1.grb ./\n', 'cp otherfile.grb ./\n',
          'cp testfile.grc ./\n', 'cp my/test/soft/dir/whizard.prc ./\n', 'echo =============================\n',
          'echo Printing content of whizard.prc \n', 'cat whizard.prc\n', 'echo =============================\n',
          'whizard --simulation_input \'write_events_file = \"myTestEvents\"\' extraTestCLIargs 2>/dev/null\n',
          'declare -x appstatus=$?\n', 'exit $appstatus\n'], self)
      chmod_mock.assert_called_once_with('Whizard_myTestV1_Run_testStep12.sh',
                                          0o755)
      shell_mock.assert_called_once_with(0, 'sh -c "./Whizard_myTestV1_Run_testStep12.sh"',
                                          callbackFunction=self.wha.redirectLogOutput,
                                          bufferLimit=209715200)

  def test_runit_no_outputfile(self):
    exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True,
                    'Whizard_myTestV1_Run_testStep12.sh': True,
                    'mytestAppLOg': True, 'whizard.out': False,
                    'myTestEvents.001.stdhep': False}
    self.wha.Model = 'mytestMODEL'
    self.wha.OutputFile = 'mytestwhizardOutputFile'
    self.wha.STEP_NUMBER = 'testStep12'
    self.wha.applicationLog = 'mytestAppLOg'
    self.wha.applicationVersion = 'myTestV1'
    self.wha.debug = False
    self.wha.evttype = 'myTestEvents'
    self.wha.extraCLIarguments = 'extraTestCLIargs'
    self.wha.genlevelcuts = False
    self.wha.getProcessInFile = True
    self.wha.ignoreapperrors = False
    self.wha.optionsdict = 9834
    self.wha.useGridFiles = True
    genmodel_mock = Mock()
    self.wha.genmodel = genmodel_mock
    whizopts_mock = Mock()
    whizopts_mock.changeAndReturn.return_value = S_OK()
    whizopts_mock.toWhizardDotIn.return_value = S_OK('mytestprocessfindme')
    whizopts_mock.getAsDict.return_value = S_OK('this shouldnt be used')
    whiz_options_mock = Mock(return_value=whizopts_mock)
    getops_mock = Mock()
    getops_mock.getValue.return_value = '/my/process/list/list.txt'
    self.wha.ops = getops_mock
    method_mock = Mock()
    method_mock.getInFile.return_value = 'mywhizardTestFile.in'
    proclist_mock = Mock(return_value=method_mock)
    file_contents = [[], ['! Event sample corresponds to luminosity 4565.3',
                          'some_logging_ignoreme', 'Event generation finished. Success!']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
         patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
         patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
         patch.dict('os.environ', {}, True), \
         patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
         patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
         patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
         patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
         patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
         patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
         patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
         patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock), \
         patch('%s.os.remove' % MODULE_NAME, new=Mock(return_value=True)) as remove_mock, \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as open_mock, \
         patch('%s.WhizardAnalysis.makeWhizardDotCut1' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
         patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK())) as shell_mock:
      open_mock.side_effect = (h for h in handles)
      assertDiracFailsWith(self.wha.runIt(), 'whizard Failed to produce STDHEP file',
                            self)
      getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
      assertEqualsImproved(len(exists_mock.mock_calls), 7, self)
      assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], self)
      assertMockCalls(appstat_mock, ['Whizard myTestV1 step testStep12',
                                       'Whizard myTestV1 Failed to produce STDHEP file'], self)
      copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                         './whizardnew.in')
      self.assertFalse(genmodel_mock.called)
      whizopts_mock.changeAndReturn.assert_called_once_with(9834)
      whizopts_mock.toWhizardDotIn.assert_called_once_with('whizard.in')
      assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], self)
      assertMockCalls(open_mock, [('Whizard_myTestV1_Run_testStep12.sh', 'w'), 'mytestAppLOg'], self)
      expected_calls = [
          [call('#!/bin/sh \n'),
           call('#####################################################################\n'),
           call('# Dynamically generated script to run a production or analysis job. #\n'),
           call('#####################################################################\n'),
           call('declare -x PATH=my/test/soft/dir:$PATH\n'),
           call('declare -x LD_LIBRARY_PATH=my/test/soft/dir/lib:my/lib/path\n'),
           call('env | sort >> localEnv.log\n'), call('echo =============================\n'),
           call('echo Printing content of whizard.in \n'), call('cat whizard.in\n'),
           call('echo =============================\n'),
           call('cp  my/test/soft/dir/whizard.mdl ./\n'),
           call('ln -s LesHouches.msugra_1.in fort.71\n'), call('cp file1.grb ./\n'),
           call('cp otherfile.grb ./\n'), call('cp testfile.grc ./\n'),
           call('cp my/test/soft/dir/whizard.prc ./\n'),
           call('echo =============================\n'),
           call('echo Printing content of whizard.prc \n'), call('cat whizard.prc\n'),
           call('echo =============================\n'),
           call('whizard --simulation_input \'write_events_file = \"myTestEvents\"\' extraTestCLIargs 2>/dev/null\n'),
           call('declare -x appstatus=$?\n'), call('exit $appstatus\n')], []]
      assertEqualsImproved(len(expected_calls), len(handles), self)
      for (expected, handle) in zip(expected_calls, handles):
        assertEqualsImproved(handle.write.mock_calls, expected, self)
      chmod_mock.assert_called_once_with('Whizard_myTestV1_Run_testStep12.sh',
                                          0o755)
      shell_mock.assert_called_once_with(0, 'sh -c "./Whizard_myTestV1_Run_testStep12.sh"',
                                          callbackFunction=self.wha.redirectLogOutput,
                                          bufferLimit=209715200)
      whizopts_mock.getAsDict.assert_called_once_with()
      assertEqualsImproved(self.wha.workflow_commons['Luminosity'],
                            4565.3, self)

  def test_log_contains_error(self):
    file_contents = [[], ['! Event sample corresponds to luminosity 4565.3',
                            '*** Fatal error: mytesterror']]
    check_logfiles(file_contents, self)

  def test_log_contains_error_1(self):
    file_contents = [[], ['! Event sample corresponds to luminosity 4565.3',
                            'PYSTOP']]
    check_logfiles(file_contents, self)

  def test_log_contains_error_2(self):
    file_contents = [[], ['! Event sample corresponds to luminosity 4565.3',
                            'No matrix element available']]
    check_logfiles(file_contents, self)

  def test_log_contains_error_3(self):
    file_contents = [[], ['! Event sample corresponds to luminosity 4565.3',
                            'Floating point exception']]
    check_logfiles(file_contents, self)

  def test_makeWhizardDotCut(self):
    self.wha.genlevelcuts = {'myprocess': ['importantvalue', 'dontmissme'],
                              'key': [], 'param': ['myparam', 'electron',
                                                      'myparam']}
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True) as open_mock:
      assertDiracSucceeds(self.wha.makeWhizardDotCut1(), self)
      open_mock.assert_called_once_with('whizard.cut1', 'w')
      open_mock = open_mock()
      assertMockCalls(open_mock.write, ['process myprocess\n', '  importantvalue\n',
                                          '  dontmissme\n', 'process key\n', 'process param\n',
                                          '  myparam\n', '  electron\n', '  myparam\n'], self)
      open_mock.close.assert_called_once_with()


def check_lists_equal(list1, list2, assertobject):
  """Checks if two lists are permutations of each other."""
  assertEqualsImproved(len(list1), len(list2), assertobject)
  for element in list1:
    assertobject.assertIn(element, list2)


def check_logfiles(file_contents, assertobject):
  exists_dict = {'list.txt': True, 'LesHouches.msugra_1.in': True,
                  'Whizard_myTestV1_Run_testStep12.sh': True,
                  'mytestAppLOg': True, 'whizard.out': False}
  set_default_values(assertobject.wha)
  assertobject.wha.debug = False
  assertobject.wha.genlevelcuts = False
  assertobject.wha.getProcessInFile = True
  genmodel_mock = Mock()
  assertobject.wha.genmodel = genmodel_mock
  whizopts_mock = Mock()
  whizopts_mock.changeAndReturn.return_value = S_OK()
  whizopts_mock.toWhizardDotIn.return_value = S_OK('mytestprocessfindme')
  whizopts_mock.getAsDict.return_value = S_OK('this shouldnt be used')
  whiz_options_mock = Mock(return_value=whizopts_mock)
  getops_mock = Mock()
  getops_mock.getValue.return_value = '/my/process/list/list.txt'
  assertobject.wha.ops = getops_mock
  method_mock = Mock()
  method_mock.getInFile.return_value = 'mywhizardTestFile.in'
  proclist_mock = Mock(return_value=method_mock)
  handles = FileUtil.getMultipleReadHandles(file_contents)
  with patch('%s.getSoftwareFolder' % MODULE_NAME, new=Mock(side_effect=[S_OK('my/test/soft/dir'), S_OK('/my/test/dep/ignorethis'), S_OK('mygridfiles/folder'), S_OK('/spectra/files')])), \
       patch('%s.removeLibc' % MODULE_NAME, new=Mock(return_value=True)), \
       patch('%s.getNewLDLibs' % MODULE_NAME, new=Mock(return_value='my/lib/path')), \
       patch('%s.resolveDeps' % MODULE_NAME, new=Mock(return_value=[{'app': 'mytestdep', 'version': '4.2'}, {'app': 'gridfiles', 'version': '1.0'}, {'app': 'beam_spectra', 'version': '20.3'}])), \
       patch.dict('os.environ', {}, True), \
       patch('%s.os.listdir' % MODULE_NAME, new=Mock(side_effect=[['gridfile1.txt', 'cool/gridfile2.ppt', 'last/file/ok'], []])), \
       patch('%s.os.path.isdir' % MODULE_NAME, new=Mock(return_value=True)), \
       patch('%s.glob.glob' % MODULE_NAME, new=Mock(side_effect=[['file1.grb', 'otherfile.grb'], ['testfile.grc']])), \
       patch('%s.os.path.exists' % MODULE_NAME, new=Mock(side_effect=lambda path: exists_dict[path])) as exists_mock, \
       patch('%s.ProcessList' % MODULE_NAME, new=proclist_mock), \
       patch('%s.WhizardAnalysis.setApplicationStatus' % MODULE_NAME) as appstat_mock, \
       patch('%s.shutil.copy' % MODULE_NAME, new=Mock(return_value=True)) as copy_mock, \
       patch('%s.WhizardOptions' % MODULE_NAME, new=whiz_options_mock), \
       patch('%s.os.remove' % MODULE_NAME, new=Mock(return_value=True)) as remove_mock, \
       patch('%s.open' % MODULE_NAME, mock_open(), create=True) as open_mock, \
       patch('%s.WhizardAnalysis.makeWhizardDotCut1' % MODULE_NAME, new=Mock(return_value=S_OK())), \
       patch('%s.os.chmod' % MODULE_NAME) as chmod_mock, \
       patch('%s.shellCall' % MODULE_NAME, new=Mock(return_value=S_OK())) as shell_mock:
    open_mock.side_effect = (h for h in handles)
    assertDiracFailsWith(assertobject.wha.runIt(),
                          'Whizard Exited With Status 1', assertobject)
    getops_mock.getValue.assert_called_once_with('/ProcessList/Location', '')
    assertEqualsImproved(len(exists_mock.mock_calls), 6, assertobject)
    assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], assertobject)
    assertMockCalls(appstat_mock, ['Whizard myTestV1 step testStep12', 'whizard Exited With Status 1'],
                     assertobject)
    copy_mock.assert_called_once_with('my/test/soft/dir/mywhizardTestFile.in',
                                       './whizardnew.in')
    assertobject.assertFalse(genmodel_mock.called)
    whizopts_mock.changeAndReturn.assert_called_once_with(9834)
    whizopts_mock.toWhizardDotIn.assert_called_once_with('whizard.in')
    assertMockCalls(remove_mock, ['Whizard_myTestV1_Run_testStep12.sh', 'mytestAppLOg'], assertobject)
    assertMockCalls(open_mock, [('Whizard_myTestV1_Run_testStep12.sh', 'w'), 'mytestAppLOg'], assertobject)
    expected_calls = [[
        call('#!/bin/sh \n'),
        call('#####################################################################\n'),
        call('# Dynamically generated script to run a production or analysis job. #\n'),
        call('#####################################################################\n'),
        call('declare -x PATH=my/test/soft/dir:$PATH\n'),
        call('declare -x LD_LIBRARY_PATH=my/test/soft/dir/lib:my/lib/path\n'),
        call('env | sort >> localEnv.log\n'), call('echo =============================\n'),
        call('echo Printing content of whizard.in \n'), call('cat whizard.in\n'),
        call('echo =============================\n'), call('cp  my/test/soft/dir/whizard.mdl ./\n'),
        call('ln -s LesHouches.msugra_1.in fort.71\n'), call('cp file1.grb ./\n'),
        call('cp otherfile.grb ./\n'), call('cp testfile.grc ./\n'),
        call('cp my/test/soft/dir/whizard.prc ./\n'), call('echo =============================\n'),
        call('echo Printing content of whizard.prc \n'), call('cat whizard.prc\n'),
        call('echo =============================\n'),
        call('whizard --simulation_input \'write_events_file = \"myTestEvents\"\' extraTestCLIargs 2>/dev/null\n'),
        call('declare -x appstatus=$?\n'), call('exit $appstatus\n')], []]
    assertEqualsImproved(len(expected_calls), len(handles), assertobject)
    for (expected, handle) in zip(expected_calls, handles):
      assertEqualsImproved(handle.write.mock_calls, expected, assertobject)
    chmod_mock.assert_called_once_with('Whizard_myTestV1_Run_testStep12.sh',
                                        0o755)
    shell_mock.assert_called_once_with(0, 'sh -c "./Whizard_myTestV1_Run_testStep12.sh"',
                                        callbackFunction=assertobject.wha.redirectLogOutput,
                                        bufferLimit=209715200)
    whizopts_mock.getAsDict.assert_called_once_with()
    assertEqualsImproved(assertobject.wha.workflow_commons['Luminosity'],
                          4565.3, assertobject)


def check_runit_for_parameters(testcaseobject, whiz_options_mock, getops_mock,
                                genmodel_mock, expected_calls, handles,
                                appstat_mock, exists_mock, rename_mock,
                                chmod_mock, glob_mock, shell_mock, copy_mock,
                                remove_mock, open_mock, expected_appstat,
                                expected_exists, expected_rename, expected_chmod,
                                expected_glob, expected_shell, expected_copy,
                                expected_genmodel, expected_getops,
                                expected_remove):  # pylint: disable=unused-argument
  """ Testcaseobject : self in a test method, provides access to assert methods and the WhizardAnalysis object
  exists_replace : Method thats used to mock out os.path.exists
  """
  assertEqualsImproved(len(expected_calls), len(handles), testcaseobject)
  for (expected, handle) in zip(expected_calls, handles):
    assertEqualsImproved(handle.write.mock_calls, expected, testcaseobject)
  assertEqualsImproved(appstat_mock.mock_calls, expected_appstat, testcaseobject)
  assertEqualsImproved(exists_mock.mock_calls, expected_exists, testcaseobject)
  assertEqualsImproved(rename_mock.mock_calls, expected_rename, testcaseobject)
  assertEqualsImproved(chmod_mock.mock_calls, expected_chmod, testcaseobject)
  assertEqualsImproved(glob_mock.mock_calls, expected_glob, testcaseobject)
  assertEqualsImproved(shell_mock.mock_calls, expected_shell, testcaseobject)
  assertEqualsImproved(copy_mock.mock_calls, expected_copy, testcaseobject)
  assertEqualsImproved(genmodel_mock.mock_calls, expected_genmodel, testcaseobject)
  assertEqualsImproved(getops_mock.mock_calls, expected_getops, testcaseobject)
  assertEqualsImproved(remove_mock.mock_calls, expected_remove, testcaseobject)
  whizopts_mock = whiz_options_mock.return_value
  whizopts_mock.changeAndReturn.assert_called_once_with(9834)
  whizopts_mock.toWhizardDotIn.assert_called_once_with('whizard.in')


def set_default_values(whizard_object):
  whizard_object.Model = 'mytestMODEL'
  whizard_object.OutputFile = 'mytestwhizardOutputFile'
  whizard_object.STEP_NUMBER = 'testStep12'
  whizard_object.applicationLog = 'mytestAppLOg'
  whizard_object.applicationVersion = 'myTestV1'
  whizard_object.debug = True
  whizard_object.energy = '99TestTeV'
  whizard_object.evttype = 'myTestEvents'
  whizard_object.extraCLIarguments = 'extraTestCLIargs'
  whizard_object.genlevelcuts = True
  whizard_object.getProcessInFile = False
  whizard_object.ignoreapperrors = False
  whizard_object.optionsdict = 9834
  whizard_object.susymodel = 1
  whizard_object.useGridFiles = True


# TODO Refactor this mess
# TODO Change noapplog to have different leshouches file and no list_of_gridfiles
# TODO add case with empty deps

EXPECTED_TEST_ENVIRON = {'LUMI_LINKER': '/spectra/files/lumi_linker_000',
                          'PHOTONS_B1': '/spectra/files/photons_beam1_linker_000',
                          'PHOTONS_B2': '/spectra/files/photons_beam2_linker_000',
                          'EBEAM': '/spectra/files/ebeam_in_linker_000',
                          'PBEAM': '/spectra/files/pbeam_in_linker_000',
                          'LUMI_EE_LINKER': '/spectra/files/lumi_ee_linker_000',
                          'LUMI_EG_LINKER': '/spectra/files/lumi_eg_linker_000',
                          'LUMI_GE_LINKER': '/spectra/files/lumi_ge_linker_000',
                          'LUMI_GG_LINKER': '/spectra/files/lumi_gg_linker_000'}



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""ILCDIRAC package, implements ILC/CLIC production and application specific stuff."""
from __future__ import absolute_import
from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

import six

__RCSID__ = "$Id$"

# pylint: disable=invalid-name

# Define Version
if six.PY3:
  from pkg_resources import get_distribution, DistributionNotFound

  try:
    __version__ = get_distribution(__name__).version
    version = __version__
  except DistributionNotFound:
    # package is not installed
    version = "Unknown"
else:
  majorVersion = 32
  minorVersion = 0
  patchLevel = 0
  preVersion = 0

  version = "v%sr%s" % (majorVersion, minorVersion)
  buildVersion = "v%dr%d" % (majorVersion, minorVersion)
  if patchLevel:
    version = "%sp%s" % (version, patchLevel)
    buildVersion = "%s build %s" % (buildVersion, patchLevel)
  if preVersion:
    version = "%s-pre%s" % (version, preVersion)
    buildVersion = "%s pre %s" % (buildVersion, preVersion)


def extension_metadata():
  """Extension metadata for ILCDIRAC bootstrap."""
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "ILC-Production": "dips://voilcdiraconfig.cern.ch:9135/Configuration/Server",
          "ILC-Certification": "dips://voilcdiraconfig.cern.ch:9135/Configuration/Server",
          },
      "default_setup": "ILC-Production",
      }



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the UploadOutputData module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Workflow.Modules.UploadOutputData import UploadOutputData
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith
from Tests.Utilities.OperationsMock import createOperationsMock

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.UploadOutputData'


class UploadOutputDataTestCase(unittest.TestCase):
  """Base class for the UploadOutputData test cases."""

  def setUp(self):
    self.upod = UploadOutputData()
    # createOperationsMock is the class, so we instantiate the class
    self.upod.ops = createOperationsMock()()

  def test_applicationSpecificInputs(self):
    self.upod.step_commons['TestFailover'] = 'something'
    self.upod.workflow_commons['outputList'] = [{'appdict': True, 'myOutput': 'yes', 'outputFile': ''},
                                                 {'outputFile': True}]
    self.upod.workflow_commons['PRODUCTION_ID'] = 1834
    self.upod.workflow_commons['JOB_ID'] = 418
    self.upod.workflow_commons['outputDataFileMask'] = ['mycoollist']
    with patch('%s.getProdFilename' % MODULE_NAME, new=Mock(side_effect=['myOutputF_1', 'other_file.txt'])):
      assertDiracSucceedsWith(self.upod.applicationSpecificInputs(), 'Parameters resolved', self)
    assertEqualsImproved(self.upod.outputList, [
        {'appdict': True, 'myOutput': 'yes', 'outputFile': 'myOutputF_1'},
        {'outputFile': 'other_file.txt'}], self)

  def test_execute_resolveinput_fails(self):
    self.upod.InputData = ['blabal']
    with patch.object(self.upod, 'resolveInputVariables', new=Mock(return_value=S_ERROR('resolv_input_testerr'))):
      assertDiracFailsWith(self.upod.execute(), 'resolv_input_testerr', self)

  def test_execute_status_notok(self):
    self.upod.stepStatus = S_ERROR('test_something_went_wrong')
    assertDiracSucceedsWith(self.upod.execute(), 'No output data upload attempt', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_getcandidate_fails(self):
    self.upod.prodOutputLFNs = ['/ilc/prod/clic/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_ERROR('cand_file_not_found_testerr'))):
      assertDiracFailsWith(self.upod.execute(), 'cand_file_not_found_testerr', self)
      assertEqualsImproved(self.upod.experiment, 'CLIC', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_metadata_not_found(self):
    self.upod.prodOutputLFNs = ['/ilc/prod/ilc/sid/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_OK({}))), \
         patch.object(self.upod, 'getFileMetadata', new=Mock(return_value=S_ERROR('no_meta_test_data_err'))):
      assertDiracFailsWith(self.upod.execute(), 'no_meta_test_data_err', self)
      assertEqualsImproved(self.upod.experiment, 'ILC_SID', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_metadata_empty(self):
    self.upod.prodOutputLFNs = ['/ilc/prod/ilc/mc-dbd/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_OK({}))), \
         patch.object(self.upod, 'getFileMetadata', new=Mock(return_value=S_OK())):
      assertDiracSucceeds(self.upod.execute(), self)
      assertEqualsImproved(self.upod.experiment, 'ILC_ILD', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_nooutputse(self):
    self.upod.prodOutputLFNs = ['/ilc/prod/ilc/mc-dbd/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_OK({}))), \
         patch.object(self.upod, 'getFileMetadata', new=Mock(return_value=S_OK({'fileTestName': {'workflowSE': 'testSE', 'otherTestMetadata': True}}))), \
         patch('%s.getDestinationSEList' % MODULE_NAME, new=Mock(return_value=S_ERROR('getdestlist_testerr'))):
      assertDiracFailsWith(self.upod.execute(), 'getdestlist_testerr', self)
      assertEqualsImproved(self.upod.experiment, 'ILC_ILD', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_disabledmodule(self):
    self.upod.prodOutputLFNs = ['/ilc/prod/ilc/mc-dbd/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_OK({}))), \
         patch.object(self.upod, 'getFileMetadata', new=Mock(return_value=S_OK({'fileTestName': {'workflowSE': 'testSE', 'otherTestMetadata': True}}))), \
         patch('%s.getDestinationSEList' % MODULE_NAME, new=Mock(return_value=S_OK('myTestSE'))):
      assertDiracSucceedsWith(self.upod.execute(), 'Module is disabled by control flag', self)
      assertEqualsImproved(self.upod.experiment, 'ILC_ILD', self)

  @patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
  def test_execute_success(self):
    trans_mock = Mock()
    trans_mock.transferAndRegisterFile.return_value = S_OK('bla')
    trans_mock.transferAndRegisterFileFailover.return_value = S_OK('bla')
    self.upod.enable = True
    self.upod.jobID = 13831
    self.upod.prodOutputLFNs = ['/ilc/prod/ilc/mc-dbd/example_file']
    with patch.object(self.upod, 'getCandidateFiles', new=Mock(return_value=S_OK({}))), \
         patch.object(self.upod, 'getFileMetadata', new=Mock(return_value=S_OK({'fileTestName': {'workflowSE': 'testSE', 'otherTestMetadata': True, 'localpath': None, 'lfn': None, 'resolvedSE': None, 'filedict': None}}))), \
         patch('%s.getDestinationSEList' % MODULE_NAME, new=Mock(return_value=S_OK('myTestSE'))), \
         patch('%s.FailoverTransfer' % MODULE_NAME, new=Mock(return_value=trans_mock)):
      assertDiracSucceedsWith(self.upod.execute(), 'Output data uploaded', self)
      assertEqualsImproved(self.upod.experiment, 'ILC_ILD', self)

  def test_gettreatedoutputlist_nodata(self):
    olist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew([], olist, {'outputFile':
                                                                       '/inval/dir/myoutputfile.txt'}))
    self.assertFalse(olist)

  def test_gettreatedoutputlist_fullcase(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_gen'], myoutputlist,
                                                          {'outputFile': 'myoutputfile_gen.stdhep'}))
    self.assertTrue(myoutputlist)

  def test_gettreatedoutputlist_fullcase_2(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_sim'], myoutputlist,
                                                          {'outputFile': 'myoutputfile_sim.slcio'}))
    self.assertTrue(myoutputlist)

  def test_gettreatedoutputlist_fullcase_3(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_rec'], myoutputlist,
                                                          {'outputFile': 'myoutputfile_rec.slcio'}))
    self.assertTrue(myoutputlist)

  def test_gettreatedoutputlist_fullcase_4(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_dst_1'], myoutputlist,
                                                          {'outputFile': 'myoutputfile_dst.slcio'}))
    self.assertTrue(myoutputlist)

  def test_gettreatedoutputlist_fullcase_5(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_and_some_more_dst_1'], myoutputlist,
                                                          {'outputFile': 'myoutputfile_dst.slcio'}))
    self.assertTrue(myoutputlist)

  def test_gettreatedoutputlist_sim_fails(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_sim'], myoutputlist,
                                                          {'outputFile':
                                                            '/invalid/dir/myoutputfile_sim.slcio'}))
    self.assertFalse(myoutputlist)

  def test_gettreatedoutputlist_rec_fails(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_rec'], myoutputlist,
                                                          {'outputFile':
                                                            '/invalid/dir/myoutputfile_rec.slcio'}))
    self.assertFalse(myoutputlist)

  def test_gettreatedoutputlist_dst_fails(self):
    myoutputlist = {}
    self.assertIsNone(self.upod.getTreatedOutputlistNew(['myoutputfile_dst_1'], myoutputlist,
                                                          {'outputFile':
                                                            '/invalid/dir/myoutputfile_dst.slcio'}))
    self.assertFalse(myoutputlist)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Utility to construct production LFNs from workflow parameters according to LHCb conventions.

:author: S. Poss
:since: Jun 16, 2010
"""

from __future__ import absolute_import
__RCSID__ = "$Id$"

import datetime
import os

from DIRAC.Core.Security.ProxyInfo import getVOfromProxyGroup
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Core.Utilities.resolvePathsAndNames import getProdFilename
from ILCDIRAC.Core.Utilities.LFNPathUtilities import cleanUpLFNPath

gLogger = gLogger.getSubLogger('ProductionData')
#############################################################################


def constructProductionLFNs(paramDict):
  """Used for local testing of a workflow, a temporary measure until LFN construction is tidied.  This works using the workflow commons for on the fly construction.

  :param dict paramDict: dictionary with at least the keys ``PRODUCTION_ID``, ``JOB_ID``, ``outputList``
  :returns: S_OK with ``ProductionOutputData``, ``LogFilePath``, ``LogTargetPath``
  """
  result = checkForMandatoryKeys(paramDict, ['PRODUCTION_ID', 'JOB_ID', 'outputList'])
  if not result['OK']:
    return result

  productionID = paramDict['PRODUCTION_ID']
  jobID = paramDict['JOB_ID']
  outputList = paramDict['outputList']

  fileTupleList = []
  gLogger.verbose('outputList %s' % (outputList))
  for info in outputList:
    # Nasty check on whether the created code parameters were not updated e.g. when changing defaults in a workflow
    fileName = info['outputFile']
    # rename to take care of correct path
    fileName = getProdFilename(fileName, int(productionID), int(jobID))
    fileTupleList.append((info['outputPath'], fileName))

  # Get all LFN(s) to output data
  outputData = []
  for fileTuple in fileTupleList:
    lfn = fileTuple[0] + "/" + str(productionID).zfill(8) + "/" + str(int(jobID) // 1000).zfill(3) + "/" + fileTuple[1]
    lfn = cleanUpLFNPath(lfn)
    outputData.append(lfn)

  # Get log file path - unique for all modules

  # get First outputfile
  basePath = fileTupleList[0][0]
  # TODO adjust for ILD
  logPath = basePath + "/" + str(productionID).zfill(8) + "/LOG"

  # used for logFile upload to the LogSE
  logFilePath = [cleanUpLFNPath('%s/%s' % (logPath, str(int(jobID) // 1000).zfill(3)))]

  resLogs = getLogPath(dict(PRODUCTION_ID=productionID,
                             JOB_ID=jobID,
                             LogFilePath=logFilePath), basePath=basePath)
  if not resLogs['OK']:
    return resLogs
  logTargetPath = resLogs['Value']['LogTargetPath']
  logFilePath = resLogs['Value']['LogFilePath']

  if not outputData:
    gLogger.info('No output data LFN(s) constructed')
  else:
    gLogger.verbose('Created the following output data LFN(s):\n%s' % ('\n'.join(outputData)))
  gLogger.verbose('Log file path is:\n%s' % logFilePath[0])
  gLogger.verbose('Log target path is:\n%s' % logTargetPath[0])
  jobOutputs = {'ProductionOutputData': outputData, 'LogFilePath': logFilePath, 'LogTargetPath': logTargetPath}
  return S_OK(jobOutputs)

#############################################################################


def getLogPath(paramDict, basePath=None):
  """Can construct log file paths even if job fails e.g. no output files available.

  :param dict paramDict: dictionary with at least the keys ``PRODUCTION_ID``, ``JOB_ID``, ``LogFilePath``
  :param str basePath: Optional, base path for the log file failover, of not set LogFilePath
    from paramDict is used as a base
  :returns: S_OK with dict with LogFilePath and LogTargetPath
  """
  result = checkForMandatoryKeys(paramDict, ['PRODUCTION_ID', 'JOB_ID', 'LogFilePath'])
  if not result['OK']:
    return result

  productionID = paramDict['PRODUCTION_ID']
  jobID = paramDict['JOB_ID']
  logFileName = "%s_%s.tar" % (str(productionID).zfill(8), str(int(jobID)).zfill(4))
  if basePath:
    logTargetPath = [cleanUpLFNPath(os.path.join(basePath, "LOG", str(productionID).zfill(8), logFileName))]
  else:
    # need to built logPath from logFilePath, as it's not there, and must be as in method above
    logPathtemp = cleanUpLFNPath(paramDict['LogFilePath']).split("/")
    logPath = "/" + os.path.join(*logPathtemp[0:-1])
    logTargetPath = ['%s/%s_%s.tar' % (logPath, str(productionID).zfill(8), str(int(jobID)).zfill(3))]

  # this is not doing anything except return the same string as was passed into the function
  logFilePath = paramDict['LogFilePath']
  gLogger.verbose('Log file path is: %s' % logFilePath)
  gLogger.verbose('Log target path is: %s' % logTargetPath)
  jobOutputs = {'LogFilePath': logFilePath, 'LogTargetPath': logTargetPath}
  return S_OK(jobOutputs)

#############################################################################


def constructUserLFNs(jobID, vo, owner, outputFiles, outputPath):
  """This method is used to supplant the standard job wrapper output data policy for ILC.  The initial convention adopted for user output files is the following:

  If outputpath is not defined:
   * <vo>/user/<initial e.g. s>/<owner e.g. sposs>/<yearMonth e.g. 2010_02>/<subdir>/<fileName>
  Otherwise:
   * <vo>/user/<initial e.g. s>/<owner e.g. sposs>/<outputPath>/<fileName>

  :param int jobID: the jobID
  :param str vo: the vo of the owners proxy
  :param str owner: the username
  :param list outputFiles: the list of outputfiles found for the job
  :param str outputPath: the outputpath defined for the job
  :returns: S_OK with list of output file lfns
  """
  initial = owner[:1]
  subdir = str(jobID // 1000)
  timeTup = datetime.date.today().timetuple()
  yearMonth = '%s_%s' % (timeTup[0], str(timeTup[1]).zfill(2))
  outputLFNs = {}
  if not vo:
    res = getVOfromProxyGroup()
    if not res['OK']:
      gLogger.error('Could not get VO from CS, assuming ilc')
      vo = 'ilc'
    else:
      vo = res['Value']
  ops = Operations(vo=vo)
  lfn_prefix = ops.getValue("LFNUserPrefix", "user")
  # Strip out any leading or trailing slashes but allow fine structure
  if outputPath:
    outputPathList = outputPath.split(os.sep)
    newPath = []
    for i in outputPathList:
      if i:
        newPath.append(i)
    outputPath = os.sep.join(newPath)

  if not isinstance(outputFiles, list):
    outputFiles = [outputFiles]

  for outputFile in outputFiles:
    # strip out any fine structure in the output file specified by the user, restrict to output file names
    # the output path field can be used to describe this
    outputFile = outputFile.replace('LFN:', '')
    lfn = ''
    if outputPath:
      lfn = os.sep + os.path.join(vo, lfn_prefix, initial, owner, outputPath + os.sep + os.path.basename(outputFile))
    else:
      lfn = os.sep + os.path.join(vo, lfn_prefix, initial, owner, yearMonth, subdir,
                                  str(jobID)) + os.sep + os.path.basename(outputFile)
    outputLFNs[outputFile] = lfn

  outputData = list(outputLFNs.values())
  if outputData:
    gLogger.info('Created the following output data LFN(s):\n%s' % ('\n'.join(outputData)))
  else:
    gLogger.info('No output LFN(s) constructed')

  return S_OK(outputData)

#############################################################################
# def _makeProductionPath(JOB_ID, LFN_ROOT, typeName, mode, prodstring, log = False):
#   """ Constructs the path in the logical name space where the output
#       data for the given production will go. In
#   """
#   result = LFN_ROOT + '/' + typeName.upper() + '/' + prodstring + '/'
#   if log:
#     try:
#       jobid = int(JOB_ID)
#       jobindex = string.zfill(jobid/10000, 4)
#     except:
#       jobindex = '0000'
#     result += jobindex
#   return result

#############################################################################
# def _makeProductionLfn(JOB_ID, LFN_ROOT, filetuple, mode, prodstring):
#   """ Constructs the logical file name according to LHCb conventions.
#       Returns the lfn without 'lfn:' prepended.
#   """
#   gLogger.debug('Making production LFN for JOB_ID %s, LFN_ROOT %s, mode %s, prodstring %s for\n%s' %(JOB_ID, LFN_ROOT, mode, prodstring, str(filetuple)))
#   try:
#     jobid = int(JOB_ID)
#     jobindex = string.zfill(jobid/10000, 4)
#   except:
#     jobindex = '0000'
#   fname = filetuple[0]
#   if re.search('lfn:', fname) or re.search('LFN:', fname):
#     return fname.replace('lfn:', '').replace('LFN:', '')
#   return LFN_ROOT + '/' + filetuple[1].upper() + '/' + prodstring + '/' + jobindex + '/' + filetuple[0]


def checkForMandatoryKeys(paramDict, keys):
  """checks for mandatory keys in the paramDict.

  :param dict paramDict: dictionary to check for mandatory ``keys``
  :param list keys: list of keys that need to be in ``paramDict``
  :returns: :func:`~DIRAC.Core.Utilities.ReturnValues.S_OK`, :func:`~DIRAC.Core.Utilities.ReturnValues.S_ERROR`
  """
  for k in keys:
    if k not in paramDict:
      return S_ERROR('%s not defined' % k)
  return S_OK()


def getExperimentFromPath(logger, exampleFile, defaultExperiment):
  """returns the experiment based on the base paths defined in the CS.

  Find experiment based on the basepaths defined in Operations/Production/ExperimentBasePaths

  :param logger: logging instance
  :param str exampleFile: file to match
  :param str defaultExperiment: experiment to return if no match
  :returns: experiment string
  """
  basePaths = Operations().getOptionsDict('Production/ExperimentBasePaths')
  if not basePaths['OK']:
    logger.warn("Could not get ExperimentBasePaths", basePaths['Message'])
  else:
    for experiment, paths in basePaths['Value'].items():
      for path in paths.split(','):
        if exampleFile.startswith(path.strip()):
          logger.info("Found experiment %s for %s matching %s" % (experiment, path, exampleFile))
          return experiment

  logger.warn("Failed to determine experiment, reverting to default: %s" % defaultExperiment)
  return defaultExperiment

#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#
